#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
# define _cffi_float_complex_t   _Fcomplex    /* include <complex.h> for it */
# define _cffi_double_complex_t  _Dcomplex    /* include <complex.h> for it */
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
# define _cffi_float_complex_t   float _Complex
# define _cffi_double_complex_t  double _Complex
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/

#include "blur.c"
#include "kmeans.c"
#include "mediancut.c"
#include "mempool.c"
#include "nearest.c"
#include "pam.c"
#include "libimagequant.c"

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 46), // double()(liq_result const *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 196), // liq_result const *
/*  2 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION, 29), // int()(float, void *)
/*  4 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13), // float
/*  5 */ _CFFI_OP(_CFFI_OP_POINTER, 199), // void *
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  7 */ _CFFI_OP(_CFFI_OP_FUNCTION, 29), // int()(liq_attr const *)
/*  8 */ _CFFI_OP(_CFFI_OP_POINTER, 187), // liq_attr const *
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 10 */ _CFFI_OP(_CFFI_OP_FUNCTION, 29), // int()(liq_image const *)
/* 11 */ _CFFI_OP(_CFFI_OP_POINTER, 193), // liq_image const *
/* 12 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 13 */ _CFFI_OP(_CFFI_OP_FUNCTION, 29), // int()(liq_result const *)
/* 14 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 15 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 16 */ _CFFI_OP(_CFFI_OP_FUNCTION, 29), // int()(void)
/* 17 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 18 */ _CFFI_OP(_CFFI_OP_FUNCTION, 28), // liq_attr *()(liq_attr const *)
/* 19 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 20 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION, 28), // liq_attr *()(void *(*)(size_t), void(*)(void *))
/* 22 */ _CFFI_OP(_CFFI_OP_POINTER, 128), // void *(*)(size_t)
/* 23 */ _CFFI_OP(_CFFI_OP_POINTER, 182), // void(*)(void *)
/* 24 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 25 */ _CFFI_OP(_CFFI_OP_FUNCTION, 28), // liq_attr *()(void)
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_attr *, int)
/* 28 */ _CFFI_OP(_CFFI_OP_POINTER, 187), // liq_attr *
/* 29 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 30 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 31 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_attr *, int, int)
/* 32 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 33 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 34 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 35 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 36 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_histogram *, liq_attr *, liq_result * *)
/* 37 */ _CFFI_OP(_CFFI_OP_POINTER, 191), // liq_histogram *
/* 38 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 39 */ _CFFI_OP(_CFFI_OP_POINTER, 82), // liq_result * *
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 41 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_histogram *, liq_attr const *, liq_histogram_entry const *, int, double)
/* 42 */ _CFFI_OP(_CFFI_OP_NOOP, 37),
/* 43 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 44 */ _CFFI_OP(_CFFI_OP_POINTER, 192), // liq_histogram_entry const *
/* 45 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 46 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/* 47 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 48 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_histogram *, liq_attr const *, liq_image *)
/* 49 */ _CFFI_OP(_CFFI_OP_NOOP, 37),
/* 50 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 51 */ _CFFI_OP(_CFFI_OP_POINTER, 193), // liq_image *
/* 52 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 53 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_histogram *, liq_color, double)
/* 54 */ _CFFI_OP(_CFFI_OP_NOOP, 37),
/* 55 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // liq_color
/* 56 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 57 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 58 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_image *, int)
/* 59 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 60 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 61 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 62 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_image *, liq_attr *, liq_result * *)
/* 63 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 64 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 65 */ _CFFI_OP(_CFFI_OP_NOOP, 39),
/* 66 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 67 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_image *, liq_color)
/* 68 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 69 */ _CFFI_OP(_CFFI_OP_NOOP, 55),
/* 70 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 71 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_image *, liq_image *)
/* 72 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 73 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 74 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 75 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_image *, unsigned char *, size_t, enum liq_ownership)
/* 76 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 77 */ _CFFI_OP(_CFFI_OP_POINTER, 197), // unsigned char *
/* 78 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28), // size_t
/* 79 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // enum liq_ownership
/* 80 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 81 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_result *, double)
/* 82 */ _CFFI_OP(_CFFI_OP_POINTER, 196), // liq_result *
/* 83 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 84 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 85 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_result *, float)
/* 86 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 87 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13),
/* 88 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 89 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_result *, liq_image *, unsigned char * *)
/* 90 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 91 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 92 */ _CFFI_OP(_CFFI_OP_POINTER, 77), // unsigned char * *
/* 93 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 94 */ _CFFI_OP(_CFFI_OP_FUNCTION, 190), // liq_error()(liq_result *, liq_image *, void *, size_t)
/* 95 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 96 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 97 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 98 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 99 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 100 */ _CFFI_OP(_CFFI_OP_FUNCTION, 37), // liq_histogram *()(liq_attr const *)
/* 101 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 102 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 103 */ _CFFI_OP(_CFFI_OP_FUNCTION, 51), // liq_image *()(liq_attr const *, void * const *, int, int, double)
/* 104 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 105 */ _CFFI_OP(_CFFI_OP_POINTER, 5), // void * const *
/* 106 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 107 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 108 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 109 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 110 */ _CFFI_OP(_CFFI_OP_FUNCTION, 51), // liq_image *()(liq_attr const *, void const *, int, int, double)
/* 111 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 112 */ _CFFI_OP(_CFFI_OP_POINTER, 199), // void const *
/* 113 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 114 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 115 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 116 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 117 */ _CFFI_OP(_CFFI_OP_FUNCTION, 51), // liq_image *()(liq_attr const *, void(*)(liq_color *, int, int, void *), void *, int, int, double)
/* 118 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 119 */ _CFFI_OP(_CFFI_OP_POINTER, 162), // void(*)(liq_color *, int, int, void *)
/* 120 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 121 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 122 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 123 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14),
/* 124 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 125 */ _CFFI_OP(_CFFI_OP_FUNCTION, 194), // liq_palette const *()(liq_result *)
/* 126 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 127 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 128 */ _CFFI_OP(_CFFI_OP_FUNCTION, 5), // void *()(size_t)
/* 129 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 130 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 131 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_attr *)
/* 132 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 133 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 134 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_attr *, int(*)(float, void *), void *)
/* 135 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 136 */ _CFFI_OP(_CFFI_OP_POINTER, 3), // int(*)(float, void *)
/* 137 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 138 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 139 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_attr *, int)
/* 140 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 141 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 142 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 143 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_attr *, void(*)(liq_attr const *, char const *, void *), void *)
/* 144 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 145 */ _CFFI_OP(_CFFI_OP_POINTER, 153), // void(*)(liq_attr const *, char const *, void *)
/* 146 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 147 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 148 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_attr *, void(*)(liq_attr const *, void *), void *)
/* 149 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 150 */ _CFFI_OP(_CFFI_OP_POINTER, 158), // void(*)(liq_attr const *, void *)
/* 151 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 152 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 153 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_attr const *, char const *, void *)
/* 154 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 155 */ _CFFI_OP(_CFFI_OP_POINTER, 186), // char const *
/* 156 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 157 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 158 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_attr const *, void *)
/* 159 */ _CFFI_OP(_CFFI_OP_NOOP, 8),
/* 160 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 161 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 162 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_color *, int, int, void *)
/* 163 */ _CFFI_OP(_CFFI_OP_POINTER, 55), // liq_color *
/* 164 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 165 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 166 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 167 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 168 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_histogram *)
/* 169 */ _CFFI_OP(_CFFI_OP_NOOP, 37),
/* 170 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 171 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_image *)
/* 172 */ _CFFI_OP(_CFFI_OP_NOOP, 51),
/* 173 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 174 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_result *)
/* 175 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 176 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 177 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(liq_result *, int(*)(float, void *), void *)
/* 178 */ _CFFI_OP(_CFFI_OP_NOOP, 82),
/* 179 */ _CFFI_OP(_CFFI_OP_NOOP, 136),
/* 180 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 181 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 182 */ _CFFI_OP(_CFFI_OP_FUNCTION, 199), // void()(void *)
/* 183 */ _CFFI_OP(_CFFI_OP_NOOP, 5),
/* 184 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 185 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1), // _Bool
/* 186 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 187 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // liq_attr
/* 188 */ _CFFI_OP(_CFFI_OP_ARRAY, 55), // liq_color[256]
/* 189 */ (_cffi_opcode_t)(256),
/* 190 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // liq_error
/* 191 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // liq_histogram
/* 192 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // liq_histogram_entry
/* 193 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // liq_image
/* 194 */ _CFFI_OP(_CFFI_OP_POINTER, 195), // liq_palette const *
/* 195 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // liq_palette
/* 196 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // liq_result
/* 197 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 4), // unsigned char
/* 198 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 199 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static int _cffi_const_LIQ_OK(unsigned long long *o)
{
  int n = (LIQ_OK) <= 0;
  *o = (unsigned long long)((LIQ_OK) | 0);  /* check that LIQ_OK is an integer */
  return n;
}

static int _cffi_const_LIQ_QUALITY_TOO_LOW(unsigned long long *o)
{
  int n = (LIQ_QUALITY_TOO_LOW) <= 0;
  *o = (unsigned long long)((LIQ_QUALITY_TOO_LOW) | 0);  /* check that LIQ_QUALITY_TOO_LOW is an integer */
  return n;
}

static int _cffi_const_LIQ_VALUE_OUT_OF_RANGE(unsigned long long *o)
{
  int n = (LIQ_VALUE_OUT_OF_RANGE) <= 0;
  *o = (unsigned long long)((LIQ_VALUE_OUT_OF_RANGE) | 0);  /* check that LIQ_VALUE_OUT_OF_RANGE is an integer */
  return n;
}

static int _cffi_const_LIQ_OUT_OF_MEMORY(unsigned long long *o)
{
  int n = (LIQ_OUT_OF_MEMORY) <= 0;
  *o = (unsigned long long)((LIQ_OUT_OF_MEMORY) | 0);  /* check that LIQ_OUT_OF_MEMORY is an integer */
  return n;
}

static int _cffi_const_LIQ_ABORTED(unsigned long long *o)
{
  int n = (LIQ_ABORTED) <= 0;
  *o = (unsigned long long)((LIQ_ABORTED) | 0);  /* check that LIQ_ABORTED is an integer */
  return n;
}

static int _cffi_const_LIQ_BITMAP_NOT_AVAILABLE(unsigned long long *o)
{
  int n = (LIQ_BITMAP_NOT_AVAILABLE) <= 0;
  *o = (unsigned long long)((LIQ_BITMAP_NOT_AVAILABLE) | 0);  /* check that LIQ_BITMAP_NOT_AVAILABLE is an integer */
  return n;
}

static int _cffi_const_LIQ_BUFFER_TOO_SMALL(unsigned long long *o)
{
  int n = (LIQ_BUFFER_TOO_SMALL) <= 0;
  *o = (unsigned long long)((LIQ_BUFFER_TOO_SMALL) | 0);  /* check that LIQ_BUFFER_TOO_SMALL is an integer */
  return n;
}

static int _cffi_const_LIQ_INVALID_POINTER(unsigned long long *o)
{
  int n = (LIQ_INVALID_POINTER) <= 0;
  *o = (unsigned long long)((LIQ_INVALID_POINTER) | 0);  /* check that LIQ_INVALID_POINTER is an integer */
  return n;
}

static int _cffi_const_LIQ_UNSUPPORTED(unsigned long long *o)
{
  int n = (LIQ_UNSUPPORTED) <= 0;
  *o = (unsigned long long)((LIQ_UNSUPPORTED) | 0);  /* check that LIQ_UNSUPPORTED is an integer */
  return n;
}

static int _cffi_const_LIQ_OWN_ROWS(unsigned long long *o)
{
  int n = (LIQ_OWN_ROWS) <= 0;
  *o = (unsigned long long)((LIQ_OWN_ROWS) | 0);  /* check that LIQ_OWN_ROWS is an integer */
  return n;
}

static int _cffi_const_LIQ_OWN_PIXELS(unsigned long long *o)
{
  int n = (LIQ_OWN_PIXELS) <= 0;
  *o = (unsigned long long)((LIQ_OWN_PIXELS) | 0);  /* check that LIQ_OWN_PIXELS is an integer */
  return n;
}

static int _cffi_const_LIQ_COPY_PIXELS(unsigned long long *o)
{
  int n = (LIQ_COPY_PIXELS) <= 0;
  *o = (unsigned long long)((LIQ_COPY_PIXELS) | 0);  /* check that LIQ_COPY_PIXELS is an integer */
  return n;
}

static liq_attr * _cffi_d_liq_attr_copy(liq_attr const * x0)
{
  return liq_attr_copy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_attr_copy(PyObject *self, PyObject *arg0)
{
  liq_attr const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_attr * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_attr_copy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(28));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_attr_copy _cffi_d_liq_attr_copy
#endif

static liq_attr * _cffi_d_liq_attr_create(void)
{
  return liq_attr_create();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_attr_create(PyObject *self, PyObject *noarg)
{
  liq_attr * result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_attr_create(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(28));
  return pyresult;
}
#else
#  define _cffi_f_liq_attr_create _cffi_d_liq_attr_create
#endif

static liq_attr * _cffi_d_liq_attr_create_with_allocator(void *(* x0)(size_t), void(* x1)(void *))
{
  return liq_attr_create_with_allocator(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_attr_create_with_allocator(PyObject *self, PyObject *args)
{
  void *(* x0)(size_t);
  void(* x1)(void *);
  liq_attr * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_attr_create_with_allocator", 2, 2, &arg0, &arg1))
    return NULL;

  x0 = (void *(*)(size_t))_cffi_to_c_pointer(arg0, _cffi_type(22));
  if (x0 == (void *(*)(size_t))NULL && PyErr_Occurred())
    return NULL;

  x1 = (void(*)(void *))_cffi_to_c_pointer(arg1, _cffi_type(23));
  if (x1 == (void(*)(void *))NULL && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_attr_create_with_allocator(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(28));
  return pyresult;
}
#else
#  define _cffi_f_liq_attr_create_with_allocator _cffi_d_liq_attr_create_with_allocator
#endif

static void _cffi_d_liq_attr_destroy(liq_attr * x0)
{
  liq_attr_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_attr_destroy(PyObject *self, PyObject *arg0)
{
  liq_attr * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_attr_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_attr_destroy _cffi_d_liq_attr_destroy
#endif

static void _cffi_d_liq_attr_set_progress_callback(liq_attr * x0, int(* x1)(float, void *), void * x2)
{
  liq_attr_set_progress_callback(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_attr_set_progress_callback(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  int(* x1)(float, void *);
  void * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_attr_set_progress_callback", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (int(*)(float, void *))_cffi_to_c_pointer(arg1, _cffi_type(136));
  if (x1 == (int(*)(float, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_attr_set_progress_callback(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_attr_set_progress_callback _cffi_d_liq_attr_set_progress_callback
#endif

static int _cffi_d_liq_get_max_colors(liq_attr const * x0)
{
  return liq_get_max_colors(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_max_colors(PyObject *self, PyObject *arg0)
{
  liq_attr const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_max_colors(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_max_colors _cffi_d_liq_get_max_colors
#endif

static int _cffi_d_liq_get_max_quality(liq_attr const * x0)
{
  return liq_get_max_quality(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_max_quality(PyObject *self, PyObject *arg0)
{
  liq_attr const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_max_quality(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_max_quality _cffi_d_liq_get_max_quality
#endif

static int _cffi_d_liq_get_min_opacity(liq_attr const * x0)
{
  return liq_get_min_opacity(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_min_opacity(PyObject *self, PyObject *arg0)
{
  liq_attr const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_min_opacity(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_min_opacity _cffi_d_liq_get_min_opacity
#endif

static int _cffi_d_liq_get_min_posterization(liq_attr const * x0)
{
  return liq_get_min_posterization(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_min_posterization(PyObject *self, PyObject *arg0)
{
  liq_attr const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_min_posterization(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_min_posterization _cffi_d_liq_get_min_posterization
#endif

static int _cffi_d_liq_get_min_quality(liq_attr const * x0)
{
  return liq_get_min_quality(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_min_quality(PyObject *self, PyObject *arg0)
{
  liq_attr const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_min_quality(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_min_quality _cffi_d_liq_get_min_quality
#endif

static double _cffi_d_liq_get_output_gamma(liq_result const * x0)
{
  return liq_get_output_gamma(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_output_gamma(PyObject *self, PyObject *arg0)
{
  liq_result const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  double result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_output_gamma(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_double(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_output_gamma _cffi_d_liq_get_output_gamma
#endif

static liq_palette const * _cffi_d_liq_get_palette(liq_result * x0)
{
  return liq_get_palette(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_palette(PyObject *self, PyObject *arg0)
{
  liq_result * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_palette const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_palette(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(194));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_palette _cffi_d_liq_get_palette
#endif

static double _cffi_d_liq_get_quantization_error(liq_result const * x0)
{
  return liq_get_quantization_error(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_quantization_error(PyObject *self, PyObject *arg0)
{
  liq_result const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  double result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_quantization_error(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_double(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_quantization_error _cffi_d_liq_get_quantization_error
#endif

static int _cffi_d_liq_get_quantization_quality(liq_result const * x0)
{
  return liq_get_quantization_quality(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_quantization_quality(PyObject *self, PyObject *arg0)
{
  liq_result const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_quantization_quality(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_quantization_quality _cffi_d_liq_get_quantization_quality
#endif

static double _cffi_d_liq_get_remapping_error(liq_result const * x0)
{
  return liq_get_remapping_error(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_remapping_error(PyObject *self, PyObject *arg0)
{
  liq_result const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  double result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_remapping_error(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_double(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_remapping_error _cffi_d_liq_get_remapping_error
#endif

static int _cffi_d_liq_get_remapping_quality(liq_result const * x0)
{
  return liq_get_remapping_quality(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_remapping_quality(PyObject *self, PyObject *arg0)
{
  liq_result const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_remapping_quality(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_remapping_quality _cffi_d_liq_get_remapping_quality
#endif

static int _cffi_d_liq_get_speed(liq_attr const * x0)
{
  return liq_get_speed(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_get_speed(PyObject *self, PyObject *arg0)
{
  liq_attr const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_get_speed(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_get_speed _cffi_d_liq_get_speed
#endif

static liq_error _cffi_d_liq_histogram_add_colors(liq_histogram * x0, liq_attr const * x1, liq_histogram_entry const * x2, int x3, double x4)
{
  return liq_histogram_add_colors(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_histogram_add_colors(PyObject *self, PyObject *args)
{
  liq_histogram * x0;
  liq_attr const * x1;
  liq_histogram_entry const * x2;
  int x3;
  double x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "liq_histogram_add_colors", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(37), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_histogram *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(37), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(44), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (liq_histogram_entry const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(44), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = (double)_cffi_to_c_double(arg4);
  if (x4 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_histogram_add_colors(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_histogram_add_colors _cffi_d_liq_histogram_add_colors
#endif

static liq_error _cffi_d_liq_histogram_add_fixed_color(liq_histogram * x0, liq_color x1, double x2)
{
  return liq_histogram_add_fixed_color(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_histogram_add_fixed_color(PyObject *self, PyObject *args)
{
  liq_histogram * x0;
  liq_color x1;
  double x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_histogram_add_fixed_color", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(37), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_histogram *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(37), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(55), arg1) < 0)
    return NULL;

  x2 = (double)_cffi_to_c_double(arg2);
  if (x2 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_histogram_add_fixed_color(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static liq_error _cffi_f_liq_histogram_add_fixed_color(liq_histogram * x0, liq_color *x1, double x2)
{
  liq_error result;
  { result = liq_histogram_add_fixed_color(x0, *x1, x2); }
  return result;
}
#endif

static liq_error _cffi_d_liq_histogram_add_image(liq_histogram * x0, liq_attr const * x1, liq_image * x2)
{
  return liq_histogram_add_image(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_histogram_add_image(PyObject *self, PyObject *args)
{
  liq_histogram * x0;
  liq_attr const * x1;
  liq_image * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_histogram_add_image", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(37), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_histogram *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(37), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_histogram_add_image(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_histogram_add_image _cffi_d_liq_histogram_add_image
#endif

static liq_histogram * _cffi_d_liq_histogram_create(liq_attr const * x0)
{
  return liq_histogram_create(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_histogram_create(PyObject *self, PyObject *arg0)
{
  liq_attr const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_histogram * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_histogram_create(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(37));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_histogram_create _cffi_d_liq_histogram_create
#endif

static void _cffi_d_liq_histogram_destroy(liq_histogram * x0)
{
  liq_histogram_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_histogram_destroy(PyObject *self, PyObject *arg0)
{
  liq_histogram * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(37), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_histogram *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(37), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_histogram_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_histogram_destroy _cffi_d_liq_histogram_destroy
#endif

static liq_error _cffi_d_liq_histogram_quantize(liq_histogram * x0, liq_attr * x1, liq_result * * x2)
{
  return liq_histogram_quantize(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_histogram_quantize(PyObject *self, PyObject *args)
{
  liq_histogram * x0;
  liq_attr * x1;
  liq_result * * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_histogram_quantize", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(37), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_histogram *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(37), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (liq_result * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_histogram_quantize(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_histogram_quantize _cffi_d_liq_histogram_quantize
#endif

static liq_error _cffi_d_liq_image_add_fixed_color(liq_image * x0, liq_color x1)
{
  return liq_image_add_fixed_color(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_add_fixed_color(PyObject *self, PyObject *args)
{
  liq_image * x0;
  liq_color x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_image_add_fixed_color", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  if (_cffi_to_c((char *)&x1, _cffi_type(55), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_add_fixed_color(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
static liq_error _cffi_f_liq_image_add_fixed_color(liq_image * x0, liq_color *x1)
{
  liq_error result;
  { result = liq_image_add_fixed_color(x0, *x1); }
  return result;
}
#endif

static liq_image * _cffi_d_liq_image_create_custom(liq_attr const * x0, void(* x1)(liq_color *, int, int, void *), void * x2, int x3, int x4, double x5)
{
  return liq_image_create_custom(x0, x1, x2, x3, x4, x5);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_create_custom(PyObject *self, PyObject *args)
{
  liq_attr const * x0;
  void(* x1)(liq_color *, int, int, void *);
  void * x2;
  int x3;
  int x4;
  double x5;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_image * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;

  if (!PyArg_UnpackTuple(args, "liq_image_create_custom", 6, 6, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (void(*)(liq_color *, int, int, void *))_cffi_to_c_pointer(arg1, _cffi_type(119));
  if (x1 == (void(*)(liq_color *, int, int, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = _cffi_to_c_int(arg4, int);
  if (x4 == (int)-1 && PyErr_Occurred())
    return NULL;

  x5 = (double)_cffi_to_c_double(arg5);
  if (x5 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_create_custom(x0, x1, x2, x3, x4, x5); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(51));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_create_custom _cffi_d_liq_image_create_custom
#endif

static liq_image * _cffi_d_liq_image_create_rgba(liq_attr const * x0, void const * x1, int x2, int x3, double x4)
{
  return liq_image_create_rgba(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_create_rgba(PyObject *self, PyObject *args)
{
  liq_attr const * x0;
  void const * x1;
  int x2;
  int x3;
  double x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_image * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "liq_image_create_rgba", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(112), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(112), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = (double)_cffi_to_c_double(arg4);
  if (x4 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_create_rgba(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(51));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_create_rgba _cffi_d_liq_image_create_rgba
#endif

static liq_image * _cffi_d_liq_image_create_rgba_rows(liq_attr const * x0, void * const * x1, int x2, int x3, double x4)
{
  return liq_image_create_rgba_rows(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_create_rgba_rows(PyObject *self, PyObject *args)
{
  liq_attr const * x0;
  void * const * x1;
  int x2;
  int x3;
  double x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_image * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "liq_image_create_rgba_rows", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(8), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(8), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(105), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (void * const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(105), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  x4 = (double)_cffi_to_c_double(arg4);
  if (x4 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_create_rgba_rows(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(51));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_create_rgba_rows _cffi_d_liq_image_create_rgba_rows
#endif

static void _cffi_d_liq_image_destroy(liq_image * x0)
{
  liq_image_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_destroy(PyObject *self, PyObject *arg0)
{
  liq_image * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_image_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_image_destroy _cffi_d_liq_image_destroy
#endif

static int _cffi_d_liq_image_get_height(liq_image const * x0)
{
  return liq_image_get_height(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_get_height(PyObject *self, PyObject *arg0)
{
  liq_image const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_image const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_get_height(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_get_height _cffi_d_liq_image_get_height
#endif

static int _cffi_d_liq_image_get_width(liq_image const * x0)
{
  return liq_image_get_width(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_get_width(PyObject *self, PyObject *arg0)
{
  liq_image const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(11), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_image const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(11), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_get_width(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_get_width _cffi_d_liq_image_get_width
#endif

static liq_error _cffi_d_liq_image_quantize(liq_image * x0, liq_attr * x1, liq_result * * x2)
{
  return liq_image_quantize(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_quantize(PyObject *self, PyObject *args)
{
  liq_image * x0;
  liq_attr * x1;
  liq_result * * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_image_quantize", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(39), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (liq_result * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(39), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_quantize(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_quantize _cffi_d_liq_image_quantize
#endif

static liq_error _cffi_d_liq_image_set_background(liq_image * x0, liq_image * x1)
{
  return liq_image_set_background(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_set_background(PyObject *self, PyObject *args)
{
  liq_image * x0;
  liq_image * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_image_set_background", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_set_background(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_set_background _cffi_d_liq_image_set_background
#endif

static liq_error _cffi_d_liq_image_set_importance_map(liq_image * x0, unsigned char * x1, size_t x2, enum liq_ownership x3)
{
  return liq_image_set_importance_map(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_set_importance_map(PyObject *self, PyObject *args)
{
  liq_image * x0;
  unsigned char * x1;
  size_t x2;
  enum liq_ownership x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "liq_image_set_importance_map", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(77), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(77), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  if (_cffi_to_c((char *)&x3, _cffi_type(79), arg3) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_set_importance_map(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_set_importance_map _cffi_d_liq_image_set_importance_map
#endif

static liq_error _cffi_d_liq_image_set_memory_ownership(liq_image * x0, int x1)
{
  return liq_image_set_memory_ownership(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_image_set_memory_ownership(PyObject *self, PyObject *args)
{
  liq_image * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_image_set_memory_ownership", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_image_set_memory_ownership(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_image_set_memory_ownership _cffi_d_liq_image_set_memory_ownership
#endif

static void _cffi_d_liq_result_destroy(liq_result * x0)
{
  liq_result_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_result_destroy(PyObject *self, PyObject *arg0)
{
  liq_result * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_result_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_result_destroy _cffi_d_liq_result_destroy
#endif

static void _cffi_d_liq_result_set_progress_callback(liq_result * x0, int(* x1)(float, void *), void * x2)
{
  liq_result_set_progress_callback(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_result_set_progress_callback(PyObject *self, PyObject *args)
{
  liq_result * x0;
  int(* x1)(float, void *);
  void * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_result_set_progress_callback", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (int(*)(float, void *))_cffi_to_c_pointer(arg1, _cffi_type(136));
  if (x1 == (int(*)(float, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_result_set_progress_callback(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_result_set_progress_callback _cffi_d_liq_result_set_progress_callback
#endif

static liq_error _cffi_d_liq_set_dithering_level(liq_result * x0, float x1)
{
  return liq_set_dithering_level(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_dithering_level(PyObject *self, PyObject *args)
{
  liq_result * x0;
  float x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_set_dithering_level", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (float)_cffi_to_c_float(arg1);
  if (x1 == (float)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_set_dithering_level(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_set_dithering_level _cffi_d_liq_set_dithering_level
#endif

static void _cffi_d_liq_set_last_index_transparent(liq_attr * x0, int x1)
{
  liq_set_last_index_transparent(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_last_index_transparent(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_set_last_index_transparent", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_set_last_index_transparent(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_set_last_index_transparent _cffi_d_liq_set_last_index_transparent
#endif

static void _cffi_d_liq_set_log_callback(liq_attr * x0, void(* x1)(liq_attr const *, char const *, void *), void * x2)
{
  liq_set_log_callback(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_log_callback(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  void(* x1)(liq_attr const *, char const *, void *);
  void * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_set_log_callback", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (void(*)(liq_attr const *, char const *, void *))_cffi_to_c_pointer(arg1, _cffi_type(145));
  if (x1 == (void(*)(liq_attr const *, char const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_set_log_callback(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_set_log_callback _cffi_d_liq_set_log_callback
#endif

static void _cffi_d_liq_set_log_flush_callback(liq_attr * x0, void(* x1)(liq_attr const *, void *), void * x2)
{
  liq_set_log_flush_callback(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_log_flush_callback(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  void(* x1)(liq_attr const *, void *);
  void * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_set_log_flush_callback", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (void(*)(liq_attr const *, void *))_cffi_to_c_pointer(arg1, _cffi_type(150));
  if (x1 == (void(*)(liq_attr const *, void *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { liq_set_log_flush_callback(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_liq_set_log_flush_callback _cffi_d_liq_set_log_flush_callback
#endif

static liq_error _cffi_d_liq_set_max_colors(liq_attr * x0, int x1)
{
  return liq_set_max_colors(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_max_colors(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_set_max_colors", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_set_max_colors(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_set_max_colors _cffi_d_liq_set_max_colors
#endif

static liq_error _cffi_d_liq_set_min_opacity(liq_attr * x0, int x1)
{
  return liq_set_min_opacity(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_min_opacity(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_set_min_opacity", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_set_min_opacity(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_set_min_opacity _cffi_d_liq_set_min_opacity
#endif

static liq_error _cffi_d_liq_set_min_posterization(liq_attr * x0, int x1)
{
  return liq_set_min_posterization(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_min_posterization(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_set_min_posterization", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_set_min_posterization(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_set_min_posterization _cffi_d_liq_set_min_posterization
#endif

static liq_error _cffi_d_liq_set_output_gamma(liq_result * x0, double x1)
{
  return liq_set_output_gamma(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_output_gamma(PyObject *self, PyObject *args)
{
  liq_result * x0;
  double x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_set_output_gamma", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (double)_cffi_to_c_double(arg1);
  if (x1 == (double)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_set_output_gamma(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_set_output_gamma _cffi_d_liq_set_output_gamma
#endif

static liq_error _cffi_d_liq_set_quality(liq_attr * x0, int x1, int x2)
{
  return liq_set_quality(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_quality(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_set_quality", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_set_quality(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_set_quality _cffi_d_liq_set_quality
#endif

static liq_error _cffi_d_liq_set_speed(liq_attr * x0, int x1)
{
  return liq_set_speed(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_set_speed(PyObject *self, PyObject *args)
{
  liq_attr * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "liq_set_speed", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_attr *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_set_speed(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_set_speed _cffi_d_liq_set_speed
#endif

static int _cffi_d_liq_version(void)
{
  return liq_version();
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_version(PyObject *self, PyObject *noarg)
{
  int result;
  PyObject *pyresult;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_version(); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  (void)noarg; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  return pyresult;
}
#else
#  define _cffi_f_liq_version _cffi_d_liq_version
#endif

static liq_error _cffi_d_liq_write_remapped_image(liq_result * x0, liq_image * x1, void * x2, size_t x3)
{
  return liq_write_remapped_image(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_write_remapped_image(PyObject *self, PyObject *args)
{
  liq_result * x0;
  liq_image * x1;
  void * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "liq_write_remapped_image", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(5), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(5), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_write_remapped_image(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_write_remapped_image _cffi_d_liq_write_remapped_image
#endif

static liq_error _cffi_d_liq_write_remapped_image_rows(liq_result * x0, liq_image * x1, unsigned char * * x2)
{
  return liq_write_remapped_image_rows(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_liq_write_remapped_image_rows(PyObject *self, PyObject *args)
{
  liq_result * x0;
  liq_image * x1;
  unsigned char * * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  liq_error result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "liq_write_remapped_image_rows", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(82), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (liq_result *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(82), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(51), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (liq_image *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(51), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(92), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (unsigned char * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(92), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = liq_write_remapped_image_rows(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_deref((char *)&result, _cffi_type(190));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_liq_write_remapped_image_rows _cffi_d_liq_write_remapped_image_rows
#endif

_CFFI_UNUSED_FN
static void _cffi_checkfld__liq_attr(liq_attr *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char const * *tmp = &p->magic_header; (void)tmp; }
  { void *(* *tmp)(size_t) = &p->malloc; (void)tmp; }
  { void(* *tmp)(void *) = &p->free; (void)tmp; }
  { double *tmp = &p->target_mse; (void)tmp; }
  { double *tmp = &p->max_mse; (void)tmp; }
  { double *tmp = &p->kmeans_iteration_limit; (void)tmp; }
  (void)((p->max_colors) | 0);  /* check that 'liq_attr.max_colors' is an integer */
  (void)((p->max_histogram_entries) | 0);  /* check that 'liq_attr.max_histogram_entries' is an integer */
  (void)((p->min_posterization_output) | 0);  /* check that 'liq_attr.min_posterization_output' is an integer */
  (void)((p->min_posterization_input) | 0);  /* check that 'liq_attr.min_posterization_input' is an integer */
  (void)((p->kmeans_iterations) | 0);  /* check that 'liq_attr.kmeans_iterations' is an integer */
  (void)((p->feedback_loop_trials) | 0);  /* check that 'liq_attr.feedback_loop_trials' is an integer */
  (void)((p->last_index_transparent) | 0);  /* check that 'liq_attr.last_index_transparent' is an integer */
  (void)((p->use_contrast_maps) | 0);  /* check that 'liq_attr.use_contrast_maps' is an integer */
  (void)((p->use_dither_map) | 0);  /* check that 'liq_attr.use_dither_map' is an integer */
  (void)((p->speed) | 0);  /* check that 'liq_attr.speed' is an integer */
  (void)((p->progress_stage1) | 0);  /* check that 'liq_attr.progress_stage1' is an integer */
  (void)((p->progress_stage2) | 0);  /* check that 'liq_attr.progress_stage2' is an integer */
  (void)((p->progress_stage3) | 0);  /* check that 'liq_attr.progress_stage3' is an integer */
  { void * *tmp = &p->progress_callback; (void)tmp; }
  { void * *tmp = &p->progress_callback_user_info; (void)tmp; }
  { void * *tmp = &p->log_callback; (void)tmp; }
  { void * *tmp = &p->log_callback_user_info; (void)tmp; }
  { void * *tmp = &p->log_flush_callback; (void)tmp; }
  { void * *tmp = &p->log_flush_callback_user_info; (void)tmp; }
}
struct _cffi_align__liq_attr { char x; liq_attr y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__liq_color(liq_color *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->r) | 0);  /* check that 'liq_color.r' is an integer */
  (void)((p->g) | 0);  /* check that 'liq_color.g' is an integer */
  (void)((p->b) | 0);  /* check that 'liq_color.b' is an integer */
  (void)((p->a) | 0);  /* check that 'liq_color.a' is an integer */
}
struct _cffi_align__liq_color { char x; liq_color y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__liq_histogram_entry(liq_histogram_entry *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { liq_color *tmp = &p->color; (void)tmp; }
  (void)((p->count) | 0);  /* check that 'liq_histogram_entry.count' is an integer */
}
struct _cffi_align__liq_histogram_entry { char x; liq_histogram_entry y; };

_CFFI_UNUSED_FN
static void _cffi_checkfld__liq_palette(liq_palette *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->count) | 0);  /* check that 'liq_palette.count' is an integer */
  { liq_color(*tmp)[256] = &p->entries; (void)tmp; }
}
struct _cffi_align__liq_palette { char x; liq_palette y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "LIQ_ABORTED", (void *)_cffi_const_LIQ_ABORTED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_BITMAP_NOT_AVAILABLE", (void *)_cffi_const_LIQ_BITMAP_NOT_AVAILABLE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_BUFFER_TOO_SMALL", (void *)_cffi_const_LIQ_BUFFER_TOO_SMALL, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_COPY_PIXELS", (void *)_cffi_const_LIQ_COPY_PIXELS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_INVALID_POINTER", (void *)_cffi_const_LIQ_INVALID_POINTER, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_OK", (void *)_cffi_const_LIQ_OK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_OUT_OF_MEMORY", (void *)_cffi_const_LIQ_OUT_OF_MEMORY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_OWN_PIXELS", (void *)_cffi_const_LIQ_OWN_PIXELS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_OWN_ROWS", (void *)_cffi_const_LIQ_OWN_ROWS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_QUALITY_TOO_LOW", (void *)_cffi_const_LIQ_QUALITY_TOO_LOW, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_UNSUPPORTED", (void *)_cffi_const_LIQ_UNSUPPORTED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIQ_VALUE_OUT_OF_RANGE", (void *)_cffi_const_LIQ_VALUE_OUT_OF_RANGE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "liq_attr_copy", (void *)_cffi_f_liq_attr_copy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 18), (void *)_cffi_d_liq_attr_copy },
  { "liq_attr_create", (void *)_cffi_f_liq_attr_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 25), (void *)_cffi_d_liq_attr_create },
  { "liq_attr_create_with_allocator", (void *)_cffi_f_liq_attr_create_with_allocator, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 21), (void *)_cffi_d_liq_attr_create_with_allocator },
  { "liq_attr_destroy", (void *)_cffi_f_liq_attr_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 131), (void *)_cffi_d_liq_attr_destroy },
  { "liq_attr_set_progress_callback", (void *)_cffi_f_liq_attr_set_progress_callback, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 134), (void *)_cffi_d_liq_attr_set_progress_callback },
  { "liq_get_max_colors", (void *)_cffi_f_liq_get_max_colors, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_liq_get_max_colors },
  { "liq_get_max_quality", (void *)_cffi_f_liq_get_max_quality, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_liq_get_max_quality },
  { "liq_get_min_opacity", (void *)_cffi_f_liq_get_min_opacity, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_liq_get_min_opacity },
  { "liq_get_min_posterization", (void *)_cffi_f_liq_get_min_posterization, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_liq_get_min_posterization },
  { "liq_get_min_quality", (void *)_cffi_f_liq_get_min_quality, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_liq_get_min_quality },
  { "liq_get_output_gamma", (void *)_cffi_f_liq_get_output_gamma, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_liq_get_output_gamma },
  { "liq_get_palette", (void *)_cffi_f_liq_get_palette, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 125), (void *)_cffi_d_liq_get_palette },
  { "liq_get_quantization_error", (void *)_cffi_f_liq_get_quantization_error, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_liq_get_quantization_error },
  { "liq_get_quantization_quality", (void *)_cffi_f_liq_get_quantization_quality, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 13), (void *)_cffi_d_liq_get_quantization_quality },
  { "liq_get_remapping_error", (void *)_cffi_f_liq_get_remapping_error, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_liq_get_remapping_error },
  { "liq_get_remapping_quality", (void *)_cffi_f_liq_get_remapping_quality, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 13), (void *)_cffi_d_liq_get_remapping_quality },
  { "liq_get_speed", (void *)_cffi_f_liq_get_speed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 7), (void *)_cffi_d_liq_get_speed },
  { "liq_histogram_add_colors", (void *)_cffi_f_liq_histogram_add_colors, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 41), (void *)_cffi_d_liq_histogram_add_colors },
  { "liq_histogram_add_fixed_color", (void *)_cffi_f_liq_histogram_add_fixed_color, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 53), (void *)_cffi_d_liq_histogram_add_fixed_color },
  { "liq_histogram_add_image", (void *)_cffi_f_liq_histogram_add_image, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 48), (void *)_cffi_d_liq_histogram_add_image },
  { "liq_histogram_create", (void *)_cffi_f_liq_histogram_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 100), (void *)_cffi_d_liq_histogram_create },
  { "liq_histogram_destroy", (void *)_cffi_f_liq_histogram_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 168), (void *)_cffi_d_liq_histogram_destroy },
  { "liq_histogram_quantize", (void *)_cffi_f_liq_histogram_quantize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 36), (void *)_cffi_d_liq_histogram_quantize },
  { "liq_image_add_fixed_color", (void *)_cffi_f_liq_image_add_fixed_color, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 67), (void *)_cffi_d_liq_image_add_fixed_color },
  { "liq_image_create_custom", (void *)_cffi_f_liq_image_create_custom, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 117), (void *)_cffi_d_liq_image_create_custom },
  { "liq_image_create_rgba", (void *)_cffi_f_liq_image_create_rgba, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 110), (void *)_cffi_d_liq_image_create_rgba },
  { "liq_image_create_rgba_rows", (void *)_cffi_f_liq_image_create_rgba_rows, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 103), (void *)_cffi_d_liq_image_create_rgba_rows },
  { "liq_image_destroy", (void *)_cffi_f_liq_image_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 171), (void *)_cffi_d_liq_image_destroy },
  { "liq_image_get_height", (void *)_cffi_f_liq_image_get_height, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 10), (void *)_cffi_d_liq_image_get_height },
  { "liq_image_get_width", (void *)_cffi_f_liq_image_get_width, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 10), (void *)_cffi_d_liq_image_get_width },
  { "liq_image_quantize", (void *)_cffi_f_liq_image_quantize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 62), (void *)_cffi_d_liq_image_quantize },
  { "liq_image_set_background", (void *)_cffi_f_liq_image_set_background, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 71), (void *)_cffi_d_liq_image_set_background },
  { "liq_image_set_importance_map", (void *)_cffi_f_liq_image_set_importance_map, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 75), (void *)_cffi_d_liq_image_set_importance_map },
  { "liq_image_set_memory_ownership", (void *)_cffi_f_liq_image_set_memory_ownership, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 58), (void *)_cffi_d_liq_image_set_memory_ownership },
  { "liq_result_destroy", (void *)_cffi_f_liq_result_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 174), (void *)_cffi_d_liq_result_destroy },
  { "liq_result_set_progress_callback", (void *)_cffi_f_liq_result_set_progress_callback, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 177), (void *)_cffi_d_liq_result_set_progress_callback },
  { "liq_set_dithering_level", (void *)_cffi_f_liq_set_dithering_level, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 85), (void *)_cffi_d_liq_set_dithering_level },
  { "liq_set_last_index_transparent", (void *)_cffi_f_liq_set_last_index_transparent, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 139), (void *)_cffi_d_liq_set_last_index_transparent },
  { "liq_set_log_callback", (void *)_cffi_f_liq_set_log_callback, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 143), (void *)_cffi_d_liq_set_log_callback },
  { "liq_set_log_flush_callback", (void *)_cffi_f_liq_set_log_flush_callback, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 148), (void *)_cffi_d_liq_set_log_flush_callback },
  { "liq_set_max_colors", (void *)_cffi_f_liq_set_max_colors, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 27), (void *)_cffi_d_liq_set_max_colors },
  { "liq_set_min_opacity", (void *)_cffi_f_liq_set_min_opacity, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 27), (void *)_cffi_d_liq_set_min_opacity },
  { "liq_set_min_posterization", (void *)_cffi_f_liq_set_min_posterization, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 27), (void *)_cffi_d_liq_set_min_posterization },
  { "liq_set_output_gamma", (void *)_cffi_f_liq_set_output_gamma, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 81), (void *)_cffi_d_liq_set_output_gamma },
  { "liq_set_quality", (void *)_cffi_f_liq_set_quality, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 31), (void *)_cffi_d_liq_set_quality },
  { "liq_set_speed", (void *)_cffi_f_liq_set_speed, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 27), (void *)_cffi_d_liq_set_speed },
  { "liq_version", (void *)_cffi_f_liq_version, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_N, 16), (void *)_cffi_d_liq_version },
  { "liq_write_remapped_image", (void *)_cffi_f_liq_write_remapped_image, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 94), (void *)_cffi_d_liq_write_remapped_image },
  { "liq_write_remapped_image_rows", (void *)_cffi_f_liq_write_remapped_image_rows, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 89), (void *)_cffi_d_liq_write_remapped_image_rows },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "magic_header", offsetof(liq_attr, magic_header),
                    sizeof(((liq_attr *)0)->magic_header),
                    _CFFI_OP(_CFFI_OP_NOOP, 155) },
  { "malloc", offsetof(liq_attr, malloc),
              sizeof(((liq_attr *)0)->malloc),
              _CFFI_OP(_CFFI_OP_NOOP, 22) },
  { "free", offsetof(liq_attr, free),
            sizeof(((liq_attr *)0)->free),
            _CFFI_OP(_CFFI_OP_NOOP, 23) },
  { "target_mse", offsetof(liq_attr, target_mse),
                  sizeof(((liq_attr *)0)->target_mse),
                  _CFFI_OP(_CFFI_OP_NOOP, 46) },
  { "max_mse", offsetof(liq_attr, max_mse),
               sizeof(((liq_attr *)0)->max_mse),
               _CFFI_OP(_CFFI_OP_NOOP, 46) },
  { "kmeans_iteration_limit", offsetof(liq_attr, kmeans_iteration_limit),
                              sizeof(((liq_attr *)0)->kmeans_iteration_limit),
                              _CFFI_OP(_CFFI_OP_NOOP, 46) },
  { "max_colors", offsetof(liq_attr, max_colors),
                  sizeof(((liq_attr *)0)->max_colors),
                  _CFFI_OP(_CFFI_OP_NOOP, 198) },
  { "max_histogram_entries", offsetof(liq_attr, max_histogram_entries),
                             sizeof(((liq_attr *)0)->max_histogram_entries),
                             _CFFI_OP(_CFFI_OP_NOOP, 198) },
  { "min_posterization_output", offsetof(liq_attr, min_posterization_output),
                                sizeof(((liq_attr *)0)->min_posterization_output),
                                _CFFI_OP(_CFFI_OP_NOOP, 198) },
  { "min_posterization_input", offsetof(liq_attr, min_posterization_input),
                               sizeof(((liq_attr *)0)->min_posterization_input),
                               _CFFI_OP(_CFFI_OP_NOOP, 198) },
  { "kmeans_iterations", offsetof(liq_attr, kmeans_iterations),
                         sizeof(((liq_attr *)0)->kmeans_iterations),
                         _CFFI_OP(_CFFI_OP_NOOP, 198) },
  { "feedback_loop_trials", offsetof(liq_attr, feedback_loop_trials),
                            sizeof(((liq_attr *)0)->feedback_loop_trials),
                            _CFFI_OP(_CFFI_OP_NOOP, 198) },
  { "last_index_transparent", offsetof(liq_attr, last_index_transparent),
                              sizeof(((liq_attr *)0)->last_index_transparent),
                              _CFFI_OP(_CFFI_OP_NOOP, 185) },
  { "use_contrast_maps", offsetof(liq_attr, use_contrast_maps),
                         sizeof(((liq_attr *)0)->use_contrast_maps),
                         _CFFI_OP(_CFFI_OP_NOOP, 185) },
  { "use_dither_map", offsetof(liq_attr, use_dither_map),
                      sizeof(((liq_attr *)0)->use_dither_map),
                      _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "speed", offsetof(liq_attr, speed),
             sizeof(((liq_attr *)0)->speed),
             _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "progress_stage1", offsetof(liq_attr, progress_stage1),
                       sizeof(((liq_attr *)0)->progress_stage1),
                       _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "progress_stage2", offsetof(liq_attr, progress_stage2),
                       sizeof(((liq_attr *)0)->progress_stage2),
                       _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "progress_stage3", offsetof(liq_attr, progress_stage3),
                       sizeof(((liq_attr *)0)->progress_stage3),
                       _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "progress_callback", offsetof(liq_attr, progress_callback),
                         sizeof(((liq_attr *)0)->progress_callback),
                         _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "progress_callback_user_info", offsetof(liq_attr, progress_callback_user_info),
                                   sizeof(((liq_attr *)0)->progress_callback_user_info),
                                   _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "log_callback", offsetof(liq_attr, log_callback),
                    sizeof(((liq_attr *)0)->log_callback),
                    _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "log_callback_user_info", offsetof(liq_attr, log_callback_user_info),
                              sizeof(((liq_attr *)0)->log_callback_user_info),
                              _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "log_flush_callback", offsetof(liq_attr, log_flush_callback),
                          sizeof(((liq_attr *)0)->log_flush_callback),
                          _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "log_flush_callback_user_info", offsetof(liq_attr, log_flush_callback_user_info),
                                    sizeof(((liq_attr *)0)->log_flush_callback_user_info),
                                    _CFFI_OP(_CFFI_OP_NOOP, 5) },
  { "r", offsetof(liq_color, r),
         sizeof(((liq_color *)0)->r),
         _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "g", offsetof(liq_color, g),
         sizeof(((liq_color *)0)->g),
         _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "b", offsetof(liq_color, b),
         sizeof(((liq_color *)0)->b),
         _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "a", offsetof(liq_color, a),
         sizeof(((liq_color *)0)->a),
         _CFFI_OP(_CFFI_OP_NOOP, 197) },
  { "color", offsetof(liq_histogram_entry, color),
             sizeof(((liq_histogram_entry *)0)->color),
             _CFFI_OP(_CFFI_OP_NOOP, 55) },
  { "count", offsetof(liq_histogram_entry, count),
             sizeof(((liq_histogram_entry *)0)->count),
             _CFFI_OP(_CFFI_OP_NOOP, 198) },
  { "count", offsetof(liq_palette, count),
             sizeof(((liq_palette *)0)->count),
             _CFFI_OP(_CFFI_OP_NOOP, 198) },
  { "entries", offsetof(liq_palette, entries),
               sizeof(((liq_palette *)0)->entries),
               _CFFI_OP(_CFFI_OP_NOOP, 188) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "liq_attr", 187, _CFFI_F_CHECK_FIELDS,
    sizeof(liq_attr), offsetof(struct _cffi_align__liq_attr, y), 0, 25 },
  { "liq_color", 55, _CFFI_F_CHECK_FIELDS,
    sizeof(liq_color), offsetof(struct _cffi_align__liq_color, y), 25, 4 },
  { "liq_histogram", 191, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "liq_histogram_entry", 192, _CFFI_F_CHECK_FIELDS,
    sizeof(liq_histogram_entry), offsetof(struct _cffi_align__liq_histogram_entry, y), 29, 2 },
  { "liq_image", 193, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "liq_palette", 195, _CFFI_F_CHECK_FIELDS,
    sizeof(liq_palette), offsetof(struct _cffi_align__liq_palette, y), 31, 2 },
  { "liq_result", 196, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "liq_error", 190, _cffi_prim_int(sizeof(liq_error), ((liq_error)-1) <= 0),
    "LIQ_OK,LIQ_QUALITY_TOO_LOW,LIQ_VALUE_OUT_OF_RANGE,LIQ_OUT_OF_MEMORY,LIQ_ABORTED,LIQ_BITMAP_NOT_AVAILABLE,LIQ_BUFFER_TOO_SMALL,LIQ_INVALID_POINTER,LIQ_UNSUPPORTED" },
  { "liq_ownership", 79, _cffi_prim_int(sizeof(enum liq_ownership), ((enum liq_ownership)-1) <= 0),
    "LIQ_OWN_ROWS,LIQ_OWN_PIXELS,LIQ_COPY_PIXELS" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "liq_attr", 187 },
  { "liq_color", 55 },
  { "liq_error", 190 },
  { "liq_histogram", 191 },
  { "liq_histogram_entry", 192 },
  { "liq_image", 193 },
  { "liq_image_get_rgba_row_callback", 162 },
  { "liq_log_callback_function", 153 },
  { "liq_log_flush_callback_function", 158 },
  { "liq_palette", 195 },
  { "liq_progress_callback_function", 3 },
  { "liq_result", 196 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  61,  /* num_globals */
  7,  /* num_struct_unions */
  2,  /* num_enums */
  12,  /* num_typenames */
  NULL,  /* no includes */
  200,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__libimagequant(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__libimagequant(void) { return NULL; }
#  else
     init_libimagequant(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__libimagequant(void)
{
  return _cffi_init("imagequant._libimagequant", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_libimagequant(void)
{
  _cffi_init("imagequant._libimagequant", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
